/* textstorage.h
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2005 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not^, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: textstorage.h,v 1.3 2005/08/16 21:01:32 ralf Exp $ */

#ifndef TEXTSTORAGE_H
#define TEXTSTORAGE_H

#include "aguixdefs.h"
#include <vector>
#include <string>
#include <utility>
#include "awidth.h"

class TextStorage
{
 public:
  virtual ~TextStorage() {}
  virtual int getNrOfLines() const = 0;
  virtual int getLine( int line_nr, unsigned int offset, int len, std::string &return_line ) const = 0;
  virtual int getLine( int line_nr, unsigned int offset, std::string &return_line ) const = 0;
  virtual int getMaxLineWidth() const = 0;
  virtual int getLineWidth( int line_nr ) const = 0;
  virtual void setLineLimit( int new_limit ) = 0;
  virtual std::pair<int,int> getRealLinePair( int line_nr ) const = 0;
  virtual int findLineNr( std::pair<int,int> real_line ) const = 0;
  virtual AWidth &getAWidth() = 0;
};

class TextStorageString : public TextStorage
{
 public:
  TextStorageString( std::string content, AWidth &lencalc );
  TextStorageString( const char *buf, int len, AWidth &lencalc );
  TextStorageString( const TextStorageString &other );
  TextStorageString &operator=( const TextStorageString &other );

  int getNrOfLines() const;
  int getLine( int line_nr, unsigned int offset, int len, std::string &return_line ) const;
  int getLine( int line_nr, unsigned int offset, std::string &return_line ) const;
  int getMaxLineWidth() const;
  int getLineWidth( int line_nr ) const;
  void setLineLimit( int new_limit );
  std::pair<int,int> getRealLinePair( int line_nr ) const;
  int findLineNr( std::pair<int,int> real_line ) const;
  AWidth &getAWidth();
 private:
  void initBuffer( std::string &content, int line_limit_width = -1 );
  unsigned int getTokenLen( unsigned int start_offset );
  
  std::string buffer;                      // stores text file
  std::vector<unsigned int> line_offsets;  // stores offset for each line (can be outside of buffer
                                           // for last line)
  std::vector<std::pair<int,int> > _real_lines;
  int getLineCharacters( int line_nr ) const;
  AWidth &_lencalc;

  //TODO better remove const modifier from getLineWidth and co?
  //     instead of declaring the cache mutable?
  mutable std::vector<int> m_line_width_cached;
  int m_current_line_limit;
};

#endif
