#include "bookmarkdb.hh"
#include "bookmarkdbentry.hh"
#include <fstream>
#include <algorithm>
#include <iterator>
#include <sstream>
#include <iostream>
#include <sys/time.h>
#include "bookmarkdb_scanner.hh"
#include "datei.h"
#include <aguix/lowlevelfunc.h>

BookmarkDB::BookmarkDB( const std::string &filename ) : m_filename( filename )
{
}

BookmarkDB::~BookmarkDB()
{
}

int BookmarkDB::read()
{
    FILE *fp;
        
    try {
        fp = worker_fopen( m_filename.c_str(), "r" );
            
        if ( fp == NULL )
            throw 3;
            
        _entries.clear();

        bmdbrestart( fp );
        bmdb_readtoken();
                
        match( BMDB_BOOKMARKS );
        match( '{' );
                
        while ( bmdb_token == BMDB_ENTRY ) {
            match( BMDB_ENTRY );
            match( '{' );
                    
            BookmarkDBEntry fbe( "", "" );
            for (;;) {
                if ( bmdb_token == BMDB_NAME ) {
                    match( BMDB_NAME );
                    match( '=' );
                    if ( bmdb_token == BMDB_STRING ) {
                        fbe.setName( bmdb_token_str );
                    }
                    match( BMDB_STRING );
                    match( ';' );
                } else if ( bmdb_token == BMDB_USEPARENT ) {
                    match( BMDB_USEPARENT );
                    match( '=' );
                    if ( bmdb_token == BMDB_TRUE ) {
                        fbe.setUseParent( true );
                        match( BMDB_TRUE );
                    } else if ( bmdb_token == BMDB_FALSE ) {
                        fbe.setUseParent( false );
                        match( BMDB_FALSE );
                    } else {
                        throw 1;
                    }
                    match( ';' );
                } else {
                    break;
                }
            }
            match( '}' );
                    
            _entries.push_back( fbe );
        }
                
        match( '}' );
                
        worker_fclose( fp );
    } catch ( int e ) {
        std::cerr << "Error in line " << bmdb_linenr << " while parsing bookmarkdb config." << std::endl;
    }

    return 0;
}

int BookmarkDB::write()
{
    Datei fh;
    if ( fh.open( m_filename.c_str(), "w" ) == 0 ) {

        fh.configOpenSection( "bookmarks" );

        for ( entries_it it1 = _entries.begin();
              it1 != _entries.end();
              it1++ ) {
         
            fh.configOpenSection( "entry" );
            
            fh.configPutPairString( "name", it1->getName().c_str() );
            fh.configPutPairBool( "useparent", it1->getUseParent() );
            
            fh.configCloseSection();
        }

        fh.configCloseSection();

        fh.close();
    } else {
        return 1;
    }
    
    return 0;
}

std::list<std::string> BookmarkDB::getCats()
{
    std::list<std::string> cats;

    for ( entries_it it1 = _entries.begin();
          it1 != _entries.end();
          it1++ ) {
        std::string c = it1->getCategory();
        if ( c.length() > 0 ) {
            if ( std::find( cats.begin(), cats.end(), c ) == cats.end() )
                cats.push_back( c );
        }
    }
    return cats;
}

std::list< BookmarkDBEntry> BookmarkDB::getEntries( const std::string &cat )
{
    std::list< BookmarkDBEntry > matched_entries;

    for ( entries_it it1 = _entries.begin();
          it1 != _entries.end();
          it1++ ) {
        std::string c = it1->getCategory();
        if ( c.length() < 1 ||
             cat == "" ||
             c == cat ) {
            matched_entries.push_back( *it1 );
        }
    }
    return matched_entries;
}

void BookmarkDB::addEntry( const BookmarkDBEntry &entry )
{
    _entries.push_back( entry );
}

void BookmarkDB::delEntry( const BookmarkDBEntry &entry )
{
    entries_it it1 = std::find( _entries.begin(),
                                _entries.end(),
                                entry );
    if ( it1 != _entries.end() ) {
        _entries.erase( it1 );
    }
}

int BookmarkDB::match( int token )
{
    if ( bmdb_token == token ) {
        bmdb_readtoken();
        return 0;
    }
    throw 1;
}
